@@ -26,7 +26,8 @@ from collections import Mapping
 
 from pyudev._libudev import libudev
 from pyudev._util import (assert_bytes, property_value_to_bytes,
-                          call_handle_error_return, udev_list_iterate)
+                          call_handle_error_return, udev_list_iterate,
+                          string_to_bool)
 
 
 class Context(object):
@@ -183,22 +184,60 @@ class Enumerator(object):
                 yield device
 
 
-class NoSuchDeviceError(LookupError):
+class DeviceNotFoundError(LookupError):
     """
-    An error indicating that no :class:`Device` was found for a specific
-    path.
+    An error indicating that no :class:`Device` was found.
+
+    .. versionchanged:: 0.5
+       Renamed from ``NoSuchDeviceError`` to its current name.
+    """
+
+
+class DeviceNotFoundAtPathError(DeviceNotFoundError):
+    """
+    An :exc:`DeviceNotFoundError` indicating that no :class:`Device` was
+    found at a given path.
     """
 
     def __init__(self, sys_path):
-        LookupError.__init__(self, sys_path)
+        DeviceNotFoundError.__init__(self, sys_path)
 
     @property
     def sys_path(self):
-        """The path that caused this error"""
+        """
+        The path that caused this error as string.
+        """
         return self.args[0]
 
     def __str__(self):
-        return 'No such device: {0!r}'.format(self.sys_path)
+        return 'No device at {0!r}'.format(self.sys_path)
+
+
+class DeviceNotFoundByNameError(DeviceNotFoundError):
+    """
+    An :exc:`DeviceNotFoundError` indicating that no :class:`Device` was
+    found with a given name.
+    """
+
+    def __init__(self, subsystem, sys_name):
+        DeviceNotFoundError.__init__(self, subsystem, sys_name)
+
+    @property
+    def subsystem(self):
+        """
+        The subsystem that caused this error as string.
+        """
+        return self.args[0]
+
+    @property
+    def sys_name(self):
+        """
+        The sys name that caused this error as string.
+        """
+        return self.args[1]
+
+    def __str__(self):
+        return 'No device {0.sys_name!r} in {0.subsystem!r}'.format(self)
 
 
 class Device(Mapping):
@@ -222,7 +261,7 @@ class Device(Mapping):
     .. warning::
 
        Do **never** use object identity (``is`` operator) to compare
-       :class:`Device` objects.  :mod:`udev` may create multiple
+       :class:`Device` objects.  :mod:`pyudev` may create multiple
        :class:`Device` objects for the same device.  Instead simply compare
        devices by value using ``==`` or ``!=``.
 
@@ -246,7 +285,7 @@ class Device(Mapping):
         ``path`` is a device path as unicode or byte string.
 
         Return a :class:`Device` object for the device.  Raise
-        :exc:`NoSuchDeviceError`, if no device was found for ``path``.
+        :exc:`DeviceNotFoundAtPathError`, if no device was found for ``path``.
         """
         if not path.startswith(context.sys_path):
             path = os.path.join(context.sys_path, path.lstrip(os.sep))
@@ -266,23 +305,55 @@ class Device(Mapping):
         device inside ``sysfs`` with the mount point included.
 
         Return a :class:`Device` object for the device.  Raise
-        :exc:`NoSuchDeviceError`, if no device was found for ``sys_path``.
+        :exc:`DeviceNotFoundAtPathError`, if no device was found for ``sys_path``.
 
         .. versionchanged:: 0.4
            Raise :exc:`NoSuchDeviceError` instead of returning ``None``, if
            no device was found for ``sys_path``
+        .. versionchanged:: 0.5
+           Raise :exc:`DeviceNotFoundAtPathError` instead of
+           :exc:`NoSuchDeviceError`
         """
         if not isinstance(context, Context):
             raise TypeError('Invalid context object')
         device = libudev.udev_device_new_from_syspath(
             context._context, assert_bytes(sys_path))
         if not device:
-            raise NoSuchDeviceError(sys_path)
+            raise DeviceNotFoundAtPathError(sys_path)
+        return cls(context, device)
+
+    @classmethod
+    def from_name(cls, context, subsystem, sys_name):
+        """
+        Create a new device from a given ``subsystem`` and a given ``sys_name``:
+
+        >>> context = pyudev.Context()
+        >>> sda = pyudev.Device.from_name(context, 'block', 'sda')
+        >>> sda
+        Device(u'/sys/devices/pci0000:00/0000:00:1f.2/host0/target0:0:0/0:0:0:0/block/sda')
+        >>> sda == pyudev.Device.from_path(context, '/block/sda')
+
+        ``context`` is the :class:`Context` in which to search the device.
+        ``subsystem`` and ``sys_name`` are byte or unicode strings, which
+        denote the subsystem and the name of the device to create.
+
+        Return a :class:`Device` object for the device.  Raise
+        :exc:`DeviceNotFoundByNameError`, if no device was found with the
+        given name.
+        """
+        if not isinstance(context, Context):
+            raise TypeError('Invalid context object')
+        device = libudev.udev_device_new_from_subsystem_sysname(
+            context._context, assert_bytes(subsystem),
+            assert_bytes(sys_name))
+        if not device:
+            raise DeviceNotFoundByNameError(subsystem, sys_name)
         return cls(context, device)
 
     def __init__(self, context, _device):
         self.context = context
         self._device = _device
+        self._attributes = Attributes(self)
 
     def __del__(self):
         libudev.udev_device_unref(self._device)
@@ -336,8 +407,8 @@ class Device(Mapping):
     @property
     def sys_path(self):
         """
-        Absolute path of this device in ``sysfs`` including the mount point
-        as unicode string.
+        Absolute path of this device in ``sysfs`` including the ``sysfs``
+        mount point as unicode string.
         """
         return libudev.udev_device_get_syspath(self._device).decode(
             sys.getfilesystemencoding())
@@ -348,8 +419,9 @@ class Device(Mapping):
         Kernel device path as unicode string.  This path uniquely identifies
         a single device.
 
-        Unlike :attr:`sys_path`, this path does not contain the mount point.
-        However, the path is absolute and starts with a slash ``'/'``.
+        Unlike :attr:`sys_path`, this path does not contain the ``sysfs``
+        mount point.  However, the path is absolute and starts with a slash
+        ``'/'``.
         """
         return libudev.udev_device_get_devpath(self._device).decode(
             sys.getfilesystemencoding())
@@ -370,29 +442,75 @@ class Device(Mapping):
         return libudev.udev_device_get_sysname(self._device).decode(
             sys.getfilesystemencoding())
 
+    @property
+    def driver(self):
+        """
+        The driver name as unicode string, or ``None``, if there is no
+        driver for this device.
+        """
+        driver = libudev.udev_device_get_driver(self._device)
+        if driver:
+            return driver.decode(sys.getfilesystemencoding())
+
     @property
     def device_node(self):
         """
-        Absolute path to the device node inside the device directory
-        (including the device directory) as unicode string.
+        Absolute path to the device node of this device as unicode string or
+        ``None``, if this device doesn't have a device node.  The path
+        includes the device directory (see :attr:`Context.device_path`).
+
+        This path always points to the actual device node associated with
+        this device, and never to any symbolic links to this device node.
+        See :attr:`device_links` to get a list of symbolic links to this
+        device node.
         """
-        return libudev.udev_device_get_devnode(self._device).decode(
-            sys.getfilesystemencoding())
+        node = libudev.udev_device_get_devnode(self._device)
+        if node:
+            return node.decode(sys.getfilesystemencoding())
 
     @property
     def device_links(self):
         """
-        The device file links in the device directory, which point to this
-        device as a list of unicode strings.
+        An iterator, which yields the absolute paths (including the device
+        directory, see :attr:`Context.device_path`) of all symbolic links
+        pointing to the :attr:`device_node` of this device.  The paths are
+        unicode strings.
+
+        UDev can create symlinks to the original device node (see
+        :attr:`device_node`) inside the device directory.  This is often
+        used to assign a constant, fixed device node to devices like
+        removeable media, which technically do not have a constant device
+        node, or to map a single device into multiple device hierarchies.
+        The property provides access to all such symbolic links, which were
+        created by UDev for this device.
         """
         entry = libudev.udev_device_get_devlinks_list_entry(self._device)
         for name in udev_list_iterate(entry):
             yield name.decode(sys.getfilesystemencoding())
 
+    @property
+    def attributes(self):
+        """
+        The system attributes of this device as read-only
+        :class:`Attributes` mapping.
+
+        System attributes are basically normal files inside the the device
+        directory.  These files contain all sorts of information about the
+        device, which may not be reflected by properties.  These attributes
+        are commonly used for matching in udev rules, and can be printed
+        using ``udevadm info --attribute-walk``.
+
+        The values of these attributes are not always proper strings, and
+        can contain arbitrary bytes.
+        """
+        return self._attributes
+
     def __iter__(self):
         """
         Iterate over the names of all properties defined for this device.
-        Property names are unicode strings.
+
+        Return a generator yielding the names of all properties of this
+        device as unicode strings.
         """
         entry = libudev.udev_device_get_properties_list_entry(self._device)
         for name in udev_list_iterate(entry):
@@ -400,7 +518,7 @@ class Device(Mapping):
 
     def __len__(self):
         """
-        Return the amount of properties defined for this device.
+        Return the amount of properties defined for this device as integer.
         """
         entry = libudev.udev_device_get_properties_list_entry(self._device)
         counter = count()
@@ -443,21 +561,19 @@ class Device(Mapping):
         """
         Get the given ``property`` from this device as boolean.
 
+        A boolean property has either a value of ``'1'`` or of ``'0'``,
+        where ``'1'`` stands for ``True``, and ``'0'`` for ``False``.  Any
+        other value causes a :exc:`~exceptions.ValueError` to be raised.
+
         ``property`` is a unicode or byte string containing the name of the
         property.
 
-        Return the property value as boolean.  A property value of ``'1'``
-        is considered ``True``, a property value of ``'0'`` is considered
-        ``False``, any other property value raises a
-        :exc:`~exceptions.ValueError`.  Raise a
-        :exc:`~exceptions.ValueError`, if the property value cannot be
-        converted to an integer.
+        Return ``True``, if the property value is ``'1'`` and ``False``, if
+        the property value is ``'0'``.  Any other value raises a
+        :exc:`~exceptions.ValueError`.  Raise a :exc:`~exceptions.KeyError`,
+        if the given property is not defined for this device.
         """
-        value = self[property]
-        if value not in ('1', '0'):
-            raise ValueError('Invalid value for boolean property: '
-                             '{0!r}'.format(value))
-        return value == '1'
+        return string_to_bool(self[property])
 
     def __hash__(self):
         return hash(self.device_path)
@@ -485,3 +601,126 @@ class Device(Mapping):
 
     def __ge__(self, other):
         raise TypeError('Device not orderable')
+
+
+def _is_attribute_file(filepath):
+    """
+    Check, if ``filepath`` points to a valid udev attribute filename.
+
+    Implementation is stolen from udev source code, ``print_all_attributes``
+    in ``udev/udevadm-info.c``.  It excludes hidden files (starting with a
+    dot), the special files ``dev`` and ``uevent`` and links.
+
+    Return ``True``, if ``filepath`` refers to an attribute, ``False``
+    otherwise.
+    """
+    filename = os.path.basename(filepath)
+    return not (filename.startswith('.') or
+                filename in ('dev', 'uevent') or
+                os.path.islink(filepath))
+
+
+class Attributes(Mapping):
+    """
+    A mapping which holds udev attributes for :class:`Device` objects.
+
+    This class subclasses the ``Mapping`` ABC, providing a read-only
+    dictionary mapping attribute names to the corresponding values.
+    Therefore all well-known dicitionary methods and operators
+    (e.g. ``.keys()``, ``.items()``, ``in``) are available to access device
+    attributes.
+    """
+
+    def __init__(self, device):
+        self.device = device
+
+    @property
+    def _attribute_files(self):
+        sys_path = self.device.sys_path
+        return [fn for fn in os.listdir(sys_path) if
+                _is_attribute_file(os.path.join(sys_path, fn)) and
+                fn in self]
+
+    def __len__(self):
+        """
+        Return the amount of attributes defined.
+        """
+        return len(self._attribute_files)
+
+    def __iter__(self):
+        """
+        Iterate over all attributes defined.
+
+        Yield each attribute name as unicode string.
+        """
+        return iter(self._attribute_files)
+
+    def __contains__(self, attribute):
+        return libudev.udev_device_get_sysattr_value(
+            self.device._device, assert_bytes(attribute)) is not None
+
+    def __getitem__(self, attribute):
+        """
+        Get the given system ``attribute`` for the device.
+
+        ``attribute`` is a unicode or byte string containing the name of the
+        system attribute.
+
+        Return the attribute value as byte string, or raise a
+        :exc:`~exceptions.KeyError`, if the given attribute is not defined
+        for this device.
+        """
+        value = libudev.udev_device_get_sysattr_value(
+            self.device._device, assert_bytes(attribute))
+        if value is None:
+            raise KeyError('No such attribute: {0}'.format(attribute))
+        return value
+
+    def asstring(self, attribute):
+        """
+        Get the given ``atribute`` for the device as unicode string.
+
+        Depending on the content of the attribute, this may or may not work.
+        Be prepared to catch :exc:`~exceptions.UnicodeDecodeError`.
+
+        ``attribute`` is a unicode or byte string containing the name of the
+        attribute.
+
+        Return the attribute value as byte string.  Raise a
+        :exc:`~exceptions.KeyError`, if the given attribute is not defined
+        for this device, or :exc:`~exceptions.UnicodeDecodeError`, if the
+        content of the attribute cannot be decoded into a unicode string.
+        """
+        return self[attribute].decode(sys.getfilesystemencoding())
+
+    def asint(self, attribute):
+        """
+        Get the given ``attribute`` as integer.
+
+        ``attribute`` is a unicode or byte string containing the name of the
+        attribute.
+
+        Return the attribute value as integer. Raise a
+        :exc:`~exceptions.KeyError`, if the given attribute is not defined
+        for this device, or a :exc:`~exceptions.ValueError`, if the
+        attribute value cannot be converted to an integer.
+        """
+        return int(self.asstring(attribute))
+
+    def asbool(self, attribute):
+        """
+        Get the given ``attribute`` from this device as boolean.
+
+        A boolean attribute has either a value of ``'1'`` or of ``'0'``,
+        where ``'1'`` stands for ``True``, and ``'0'`` for ``False``.  Any
+        other value causes a :exc:`~exceptions.ValueError` to be raised.
+
+        ``attribute`` is a unicode or byte string containing the name of the
+        attribute.
+
+        Return ``True``, if the attribute value is ``'1'`` and ``False``, if
+        the attribute value is ``'0'``.  Any other value raises a
+        :exc:`~exceptions.ValueError`.  Raise a :exc:`~exceptions.KeyError`,
+        if the given attribute is not defined for this device.
+        """
+        return string_to_bool(self.asstring(attribute))
