@@ -100,13 +100,17 @@ SIGNATURES = {
         ref=([udev_device_p], udev_device_p),
         unref=([udev_device_p], None),
         new_from_syspath=([udev_p, c_char_p], udev_device_p),
+        new_from_subsystem_sysname=([udev_p, c_char_p, c_char_p],
+                                    udev_device_p),
         get_parent=([udev_device_p], udev_device_p),
         get_devpath=([udev_device_p], c_char_p),
         get_subsystem=([udev_device_p], c_char_p),
         get_syspath=([udev_device_p], c_char_p),
         get_sysname=([udev_device_p], c_char_p),
+        get_driver=([udev_device_p], c_char_p),
         get_devnode=([udev_device_p], c_char_p),
         get_property_value=([udev_device_p, c_char_p], c_char_p),
+        get_sysattr_value=([udev_device_p, c_char_p], c_char_p),
         get_action=([udev_device_p], c_char_p),
         get_devlinks_list_entry=([udev_device_p], udev_list_entry_p),
         get_properties_list_entry=([udev_device_p], udev_list_entry_p)),
