@@ -16,6 +16,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+from __future__ import unicode_literals
 
 import sys
 import os
@@ -44,38 +45,32 @@ def test_from_netlink_invalid_source(context):
 
 def _assert_from_netlink_called(context, *args):
     with py.test.patch_libudev('udev_monitor_new_from_netlink') as func:
-        source = args[0] if args else 'udev'
+        source = args[0].encode('ascii') if args else b'udev'
         func.return_value = mock.sentinel.pointer
         monitor = Monitor.from_netlink(context, *args)
         func.assert_called_with(context._context, source)
-        assert isinstance(func.call_args[0][1], str)
+        assert isinstance(func.call_args[0][1], bytes)
         assert monitor._monitor is mock.sentinel.pointer
 
 
 def test_from_netlink_source_udev(context):
     monitor = Monitor.from_netlink(context)
     assert monitor._monitor
-    monitor = Monitor.from_netlink(context, source=b'udev')
-    assert monitor._monitor
-    monitor = Monitor.from_netlink(context, source=u'udev')
+    monitor = Monitor.from_netlink(context, source='udev')
     assert monitor._monitor
 
 
 def test_from_netlink_source_udev_mock(context):
     _assert_from_netlink_called(context)
-    _assert_from_netlink_called(context, b'udev')
-    _assert_from_netlink_called(context, u'udev')
+    _assert_from_netlink_called(context, 'udev')
 
 
 def test_from_netlink_source_kernel(context):
-    monitor = Monitor.from_netlink(context, source=b'kernel')
-    assert monitor._monitor
-    monitor = Monitor.from_netlink(context, source=u'kernel')
+    monitor = Monitor.from_netlink(context, source='kernel')
     assert monitor._monitor
 
 
 def test_from_netlink_source_kernel_mock(context):
-    _assert_from_netlink_called(context, b'kernel')
     _assert_from_netlink_called(context, 'kernel')
 
 
@@ -88,11 +83,12 @@ def test_from_socket_mock(context, socket_path):
     with py.test.patch_libudev('udev_monitor_new_from_socket') as func:
         func.return_value = mock.sentinel.pointer
         monitor = Monitor.from_socket(context, str(socket_path))
-        func.assert_called_with(context._context, str(socket_path))
+        func.assert_called_with(context._context, str(socket_path).encode(
+            sys.getfilesystemencoding()))
         assert monitor._monitor is mock.sentinel.pointer
-        Monitor.from_socket(context, u'foobar')
-        func.assert_called_with(context._context, 'foobar')
-        assert isinstance(func.call_args[0][1], str)
+        Monitor.from_socket(context, 'foobar')
+        func.assert_called_with(context._context, b'foobar')
+        assert isinstance(func.call_args[0][1], bytes)
 
 
 def test_fileno(monitor):
@@ -114,7 +110,7 @@ def test_filter_by_no_subsystem(monitor):
 
 def test_filter_by_subsystem_no_dev_type(monitor):
     monitor.filter_by(b'input')
-    monitor.filter_by(u'input')
+    monitor.filter_by('input')
 
 
 def test_filter_by_subsystem_no_dev_type_mock(monitor):
@@ -123,14 +119,14 @@ def test_filter_by_subsystem_no_dev_type_mock(monitor):
         func.return_value = 0
         monitor.filter_by(b'input')
         func.assert_called_with(monitor._monitor, b'input', None)
-        monitor.filter_by(u'input')
+        monitor.filter_by('input')
         func.assert_called_with(monitor._monitor, b'input', None)
-        assert isinstance(func.call_args[0][1], str)
+        assert isinstance(func.call_args[0][1], bytes)
 
 
 def test_filter_by_subsystem_dev_type(monitor):
     monitor.filter_by('input', b'usb_interface')
-    monitor.filter_by('input', u'usb_interface')
+    monitor.filter_by('input', 'usb_interface')
 
 
 def test_filter_by_subsystem_dev_type_mock(monitor):
@@ -138,10 +134,10 @@ def test_filter_by_subsystem_dev_type_mock(monitor):
     with py.test.patch_libudev(epic_func_name) as func:
         func.return_value = 0
         monitor.filter_by(b'input', b'usb_interface')
-        func.assert_called_with(monitor._monitor, 'input', 'usb_interface')
-        monitor.filter_by(u'input', u'usb_interface')
-        func.assert_called_with(monitor._monitor, 'input', 'usb_interface')
-        assert isinstance(func.call_args[0][2], str)
+        func.assert_called_with(monitor._monitor, b'input', b'usb_interface')
+        monitor.filter_by('input', 'usb_interface')
+        func.assert_called_with(monitor._monitor, b'input', b'usb_interface')
+        assert isinstance(func.call_args[0][2], bytes)
 
 
 def test_filter_by_memory_error_mock(monitor):
@@ -216,7 +212,6 @@ def test_enable_receiving_error_mock(context, monitor, socket_path):
             assert error.filename == str(socket_path)
 
 
-@py.test.mark.skipif("not config.getvalue('allow_privileges')")
 @py.test.mark.privileged
 def test_receive_device(monitor):
     # forcibly unload the dummy module to avoid hangs
@@ -243,8 +238,8 @@ def test_receive_device_mock(monitor):
             func.return_value = mock.sentinel.pointer
             actfunc.return_value = b'action'
             action, device = monitor.receive_device()
-            assert action == u'action'
-            assert isinstance(action, unicode)
+            assert action == 'action'
+            assert py.test.is_unicode_string(action)
             assert isinstance(device, Device)
             assert device.context is monitor.context
             assert device._device is mock.sentinel.pointer
@@ -268,7 +263,6 @@ def test_receive_device_error_mock(monitor):
             assert error.strerror == os.strerror(errno.ENOENT)
 
 
-@py.test.mark.skipif("not config.getvalue('allow_privileges')")
 @py.test.mark.privileged
 def test_iter(monitor):
     py.test.unload_dummy()
